/* Copyright 2017 Leandro Augusto Frata Fernandes.
 * Distributed under the GNU General Public License, Version 3.
 *
 * Last update: August 8th, 2017.
 *
 * This implementation is part of the material accompanying the book:
 *
 *    lgebra Geomtrica e Aplicaes
 *    Fernandes, L. A. F.; Lavor, C., Oliveira, M. M.
 *
 *    Sociedade Brasileira de Matemtica Aplicada e Computacional (SBMAC)
 *    Notas em Matemtica Aplicada (NoMA), Volume 85, 2017
 *
 *    ISSN: 2175-3385 | e-ISSN: 2236-5915
 *    ISBN: 978-85-8215-081-8 | e-ISBN: 978-85-8215-080-1
 *
 *    http://www.ic.uff.br/algebrageometrica
 */

#include <ga.hpp>
#include <array>
#include <iostream>

using namespace ga;

int main() {
	{
		auto e1 = e(1);
		auto e2 = e(2);
		auto e3 = e(3);

		std::cout << "(e1 + e2) ^ (e3 + e2) = " << ((e1  + e2) ^ (e3 + e2)) << std::endl;
		std::cout << "(e2 - e1) ^ (e1 - 2 * e3) = " << ((e2  - e1) ^ (e1 - 2 * e3)) << std::endl;
		std::cout << "(4 * e1 + e2 + e3) ^ (3 * e1) = " << ((4 * e1 + e2 + e3) ^ (3 * e1)) << std::endl;
		std::cout << "(e2 + e3) ^ (1.0/2.0 * e1 + e2 + 3.0/2.0 * e3) = " << ((e2 + e3) ^ (1.0/2.0 * e1 + e2 + 3.0/2.0 * e3)) << std::endl;
		std::cout << "(e1 + e2) ^ ((e2 ^ e1) + (e3 ^ e2)) = " << ((e1 + e2) ^ ((e2 ^ e1) + (e3 ^ e2))) << std::endl;
		std::cout << std::endl;
	}
	
	{
		auto e1 = e(1);
		auto e2 = e(2);
		auto e3 = e(3);
		auto a = e1 + e3;
		auto b = e1 + e2;
		signed_metric<3, 0> r30;
		
		std::cout << "a = " << a << std::endl;
		std::cout << "b = " << b << std::endl;
		std::cout << "scp(a, b, r30) = " << scp(a, b, r30) << std::endl;
		std::cout << "lcont(e3, b, r30) = " << lcont(e3, b, r30) << std::endl;
		std::cout << "lcont(e3, (a ^ b), r30) = " << lcont(e3, (a ^ b), r30) << std::endl;
		std::cout << "lcont((a ^ b), e1, r30) = " << lcont((a ^ b), e1, r30) << std::endl;
		std::cout << "scp(2 * a + b, a + b, r30) = " << scp(2 * a + b, a + b, r30) << std::endl;
		std::cout << "rcont(e1 ^ e2 ^ e3, b, r30) = " << rcont(e1 ^ e2 ^ e3, b, r30) << std::endl;
		std::cout << std::endl;
	}
	
	{
		auto e1 = e(1);
		auto e2 = e(2);
		auto e3 = e(3);
		auto e4 = e(4);
		signed_metric<3, 1> r31;
		
		std::array<multivector, 4> a_list = {e1, 5 * e2, 2 * e2 + e3, e4};
		std::array<multivector, 2> b_list = {e1 ^ (e2 + 2 * e3) ^ e4, e1 ^ e2 ^ e3};
		
		for (auto a : a_list) {
			for (auto b : b_list) {
				auto lhs1 = a ^ b;
				auto rhs1 = 0.5 * (gp(a, b, r31) + gp(involution(b), a, r31));
				std::cout << lhs1 << " = " << rhs1 << " (" << (equal(lhs1, rhs1) ? "equal" : "different" ) << ")" << std::endl;
				
				auto lhs2 = b ^ a;
				auto rhs2 = 0.5 * (gp(b, a, r31) + gp(a, involution(b), r31));
				std::cout << lhs2 << " = " << rhs2 << " (" << (equal(lhs2, rhs2) ? "equal" : "different" ) << ")" << std::endl;
				
				auto lhs3 = lcont(a, b, r31);
				auto rhs3 = 0.5 * (gp(a, b, r31) - gp(involution(b), a, r31));
				std::cout << lhs3 << " = " << rhs3 << " (" << (equal(lhs3, rhs3) ? "equal" : "different" ) << ")" << std::endl;
				std::cout << std::endl;
			}
		}
	}
	
	return 0;
}
