# Copyright 2017 Leandro Augusto Frata Fernandes.
# Distributed under the GNU General Public License, Version 3.
#
# Last update: August 9th, 2017.
#
# This implementation is part of the material accompanying the book:
#
#    Álgebra Geométrica e Aplicações
#    Fernandes, L. A. F.; Lavor, C.; Oliveira, M. M.
#
#    Sociedade Brasileira de Matemática Aplicada e Computacional (SBMAC)
#    Notas em Matemática Aplicada (NoMA), Volume 85, 2017
#
#    ISSN: 2175-3385 | e-ISSN: 2236-5915
#    ISBN: 978-85-8215-081-8 | e-ISBN: 978-85-8215-080-1
#
#    http://www.ic.uff.br/algebrageometrica

from ga import *

e1 = e(1)
e2 = e(2)
e3 = e(3)

print("(e1 + e2) ^ (e3 + e2) =", ((e1 + e2) ^ (e3 + e2)))
print("(e2 - e1) ^ (e1 - 2 * e3) =", ((e2 - e1) ^ (e1 - 2 * e3)))
print("(4 * e1 + e2 + e3) ^ (3 * e1) =", ((4 * e1 + e2 + e3) ^ (3 * e1)))
print("(e2 + e3) ^ (1.0/2.0 * e1 + e2 + 3.0/2.0 * e3) =", ((e2 + e3) ^ (1.0 / 2.0 * e1 + e2 + 3.0 / 2.0 * e3)))
print("(e1 + e2) ^ ((e2 ^ e1) + (e3 ^ e2)) =", ((e1 + e2) ^ ((e2 ^ e1) + (e3 ^ e2))))
print()

e1 = e(1)
e2 = e(2)
e3 = e(3)
a = e1 + e3
b = e1 + e2
r30 = SignedMetric(3, 0)

print("a =", a)
print("b =", b)
print("scp(a, b, r30) =", scp(a, b, r30))
print("lcont(e3, b, r30) =", lcont(e3, b, r30))
print("lcont(e3, (a ^ b), r30) =", lcont(e3, (a ^ b), r30))
print("lcont((a ^ b), e1, r30) =", lcont((a ^ b), e1, r30))
print("scp(2 * a + b, a + b, r30) =", scp(2 * a + b, a + b, r30))
print("rcont(e1 ^ e2 ^ e3, b, r30) =", rcont(e1 ^ e2 ^ e3, b, r30))
print()

e1 = e(1)
e2 = e(2)
e3 = e(3)
e4 = e(4)
r31 = SignedMetric(3, 1)


def check(lhs, rhs):
    if equal(lhs, rhs):
        return "(equal)"
    else:
        return "(different)"

a_list = [e1, 5 * e2, 2 * e2 + e3, e4]
b_list = [e1 ^ (e2 + 2 * e3) ^ e4, e1 ^ e2 ^ e3]

for a in a_list:
    for b in b_list:
        lhs1 = a ^ b
        rhs1 = 0.5 * (gp(a, b, r31) + gp(involution(b), a, r31))
        print(lhs1, "=", rhs1, check(lhs1, rhs1))

        lhs2 = b ^ a
        rhs2 = 0.5 * (gp(b, a, r31) + gp(a, involution(b), r31))
        print(lhs2, "=", rhs2, check(lhs2, rhs2))

        lhs3 = lcont(a, b, r31)
        rhs3 = 0.5 * (gp(a, b, r31) - gp(involution(b), a, r31))
        print(lhs3, "=", rhs3, check(lhs3, rhs3))
        print()
